# Generated by Django 2.1.3 on 2019-01-21 16:49

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [("mds", "0002_area_color")]

    operations = [
        migrations.AddField(
            model_name="eventrecord",
            name="timestamp",
            field=models.DateTimeField(
                db_index=True, default=django.utils.timezone.now
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="eventrecord",
            name="saved_at",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterUniqueTogether(
            name="eventrecord", unique_together={("device", "timestamp")}
        ),
    ]
