# Generated by Django 2.1.11 on 2019-08-22 12:39

from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [("mds", "0038_eventrecord_event_type_reason")]

    operations = [
        migrations.AlterField(
            model_name="eventrecord",
            name="event_type",
            field=mds.models.UnboundedCharField(
                choices=[
                    ("register", "Register"),
                    ("service_start", "Service start"),
                    ("service_end", "Service end"),
                    ("provider_drop_off", "Provider drop off"),
                    ("provider_pick_up", "Provider pick up"),
                    ("city_pick_up", "City pick up"),
                    ("reserve", "Reserve"),
                    ("cancel_reservation", "Cancel reservation"),
                    ("trip_start", "Trip start"),
                    ("trip_enter", "Trip enter"),
                    ("trip_leave", "Trip leave"),
                    ("trip_end", "Trip end"),
                    ("deregister", "Deregister"),
                    ("telemetry", "Received telemetry"),
                    ("battery_charged", "Battery charged"),
                    ("rebalance_drop_off", "Rebalance drop off"),
                    ("maintenance_drop_off", "Maintenance drop off"),
                    ("low_battery", "Low battery"),
                    ("maintenance", "Maintenance"),
                    ("rebalance_pick_up", "Rebalance pick up"),
                    ("maintenance_pick_up", "Maintenance pick up"),
                ]
            ),
        )
    ]
