# Generated by Django 2.1.4 on 2019-01-17 16:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("mds", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="area",
            name="color",
            field=models.CharField(
                default="#FFFFFF", help_text="hexa representation", max_length=8
            ),
        )
    ]
