# Generated by Django 2.2 on 2019-07-03 09:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("mds", "0031_post_eventrecord_saved_at_index")]

    operations = [
        migrations.AddField(
            model_name="device",
            name="saved_at",
            field=models.DateTimeField(db_index=True, null=True),
        )
    ]
