# Generated by Django 2.1.7 on 2019-03-28 16:19

from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0016_agency_api_configuration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='device',
            name='category',
            field=mds.models.UnboundedCharField(choices=[('bicycle', 'Bicycle'), ('scooter', 'Scooter'), ('car', 'Car')]),
        ),
        migrations.AlterField(
            model_name='provider',
            name='device_category',
            field=mds.models.UnboundedCharField(choices=[('bicycle', 'Bicycle'), ('scooter', 'Scooter'), ('car', 'Car')]),
        ),
    ]
