# Generated by Django 2.2 on 2019-06-19 14:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("mds", "0028_pre_provider_operator")]

    operations = [
        migrations.RunSQL(
            """
            UPDATE mds_provider SET operator = (
                CASE WHEN name IN ('LADOT', 'LADOT Sandbox') THEN false
                ELSE true
                END
            )
            """,
            migrations.RunSQL.noop,
        )
    ]
