# Generated by Django 2.2 on 2019-05-17 08:45

import django.contrib.postgres.fields.jsonb
from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [("mds", "0022_provider_colors")]

    operations = [
        migrations.AlterField(
            model_name="provider",
            name="agency_api_authentication",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                default=mds.models.agency_api_authentication_default,
                verbose_name="Agency API Authentication",
            ),
        ),
        migrations.AlterField(
            model_name="provider",
            name="agency_api_configuration",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                default=mds.models.agency_api_configuration_default,
                verbose_name="Agency API Configuration",
            ),
        ),
        migrations.AlterField(
            model_name="provider",
            name="api_authentication",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                default=mds.models.provider_api_authentication_default,
                verbose_name="Provider API Authentication",
            ),
        ),
        migrations.AlterField(
            model_name="provider",
            name="api_configuration",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True,
                default=mds.models.provider_api_configuration_default,
                verbose_name="Provider API Configuration",
            ),
        ),
    ]
