# Generated by Django 2.2 on 2019-05-23 13:18

import django.contrib.postgres.fields.jsonb
from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0024_provider_device_category_deprecation'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='provider',
            name='device_category',
        ),
        migrations.AlterField(
            model_name='provider',
            name='colors',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, validators=[mds.models.colors_validator], verbose_name='colors for distinguishing provider icons'),
        ),
    ]
