# Generated by Django 2.1.7 on 2019-03-25 13:49

from django.db import migrations
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0017_change_bike_to_bicycle_in_choices_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='provider',
            name='base_api_url',
            field=mds.models.UnboundedCharField(blank=True, default='', verbose_name='Base Provider API URL'),
        ),
        migrations.AlterField(
            model_name='provider',
            name='oauth2_url',
            field=mds.models.UnboundedCharField(blank=True, default='', verbose_name='OAuth2 URL (if different)'),
        ),
    ]
