# Generated by Django 2.2 on 2019-06-19 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("mds", "0029_post_provider_operator_fill")]

    operations = [
        migrations.AlterField(
            model_name="provider",
            name="operator",
            field=models.BooleanField(default=True),
        )
    ]
