# Generated by Django 2.1.3 on 2019-04-18 13:49

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import mds.models


class Migration(migrations.Migration):

    dependencies = [
        ('mds', '0019_delete_creation_deletion_date_fields_in_area_and_polygon'),
    ]

    operations = [
        migrations.AlterField(
            model_name='area',
            name='color',
            field=mds.models.UnboundedCharField(default='#FFFFFF', help_text='hexa representation'),
        ),
        migrations.AlterField(
            model_name='device',
            name='dn_battery_pct',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='dn_gps_point',
            field=django.contrib.gis.db.models.fields.PointField(blank=True, null=True, srid=4326),
        ),
        migrations.AlterField(
            model_name='device',
            name='dn_gps_timestamp',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='dn_status',
            field=mds.models.UnboundedCharField(choices=[('available', 'Available'), ('reserved', 'Reserved'), ('unavailable', 'Unavailable'), ('removed', 'Removed'), ('trip', 'Trip'), ('elsewhere', 'Elsewhere'), ('inactive', 'Inactive'), ('unknown', 'Unknown')], default='unknown'),
        ),
        migrations.AlterField(
            model_name='device',
            name='manufacturer',
            field=mds.models.UnboundedCharField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='device',
            name='model',
            field=mds.models.UnboundedCharField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='device',
            name='year_manufactured',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='eventrecord',
            name='point',
            field=django.contrib.gis.db.models.fields.PointField(blank=True, null=True, srid=4326),
        ),
        migrations.AlterField(
            model_name='eventrecord',
            name='source',
            field=mds.models.UnboundedCharField(choices=[('push', 'push'), ('pull', 'pull')], default='push'),
        ),
        migrations.AlterField(
            model_name='provider',
            name='name',
            field=mds.models.UnboundedCharField(blank=True, default=''),
        ),
    ]
